import click
import os
from .db import connect, genHash, verify
from cryptography.exceptions import InvalidKey
from .messages import Error, Success
from .constant import secret_salt

@click.command()
def setup():
    if not click.confirm(f"Do you want to create an account on 'master'?"):
        click.echo("Aborted")
        return
    
    p = click.prompt("Your master password", hide_input=True)
    hashpw = genHash(p)
    
    try:
        c, conn = connect()
        c.execute(""" CREATE TABLE IF NOT EXISTS user (
                        id INTEGER PRIMARY KEY AUTOINCREMENT, 
                        account varchar(255) UNIQUE,
                        username varchar(255),
                        password varchar(255)
                    ); """)
        c.execute(""" 
                    INSERT INTO user (account, username, password)
                    VALUES ('master', ?, ?);
                """, (secret_salt, hashpw,))
        conn.commit()
        conn.close()
        Success("Successful setup on 'master'")
    except:
        Error("An error occurred. Check if you already have setup")


@click.command()
def reset():

    op = click.prompt("Old password", hide_input=True)
    verify(op)

    try:
        c, conn = connect()
        np = click.prompt("New password", hide_input=True)
        hashed_np = genHash(np)
        c.execute(""" 
                    UPDATE user
                    SET password=? 
                    WHERE account = 'master'
                    """,
                    (hashed_np,))
        conn.commit()
        conn.close()
        Success()
    except:
        Error()

@click.command()
def wipe():
    """ Removes all data, all passwords from device (irreversable) """
    if not click.confirm(f"Do you want to continue to delete ALL information on table?"):
        click.echo(click.style('Aborted', fg='red'))
        return

    p = click.prompt("Confirm with password", hide_input=True)
    verify(p)

    try:
        c, conn = connect()
        c.execute(f""" 
                DROP TABLE user;
                """)
        conn.commit()
        conn.close()
        Success("Successfully deleted table.")
    except:
        Error()





import click
import os
from .db import connect, verify
from .encryption import encrypt, decrypt
from .messages import Error, Success
from cryptography.exceptions import InvalidKey

@click.command()
@click.password_option()
def add(password:str):
    verify(password)
    account = click.prompt("Account (gmail, reddit, etc)").lower()
    username = click.prompt(f"Account username for {account}")
    account_password = click.prompt("Account password", hide_input=True)

    account_password = encrypt(password, account_password)

    try:
        c, conn = connect()
        c.execute("""
            INSERT INTO user (account, username, password)
            VALUES (?, ?, ?);
        """,
        (account, username, account_password))
        conn.commit()
        conn.close()
        Success()
    except:
        Error("An error occurred. Check if an account already exists")
    
@click.command()
@click.password_option()
def update(password:str):
    verify(password)
    account = click.prompt("Account (gmail, reddit, etc)").lower()
    np = click.prompt(f"New password for {account}", hide_input=True)

    np = encrypt(password, np)

    try:
        c, conn = connect()
        c.execute("""
            UPDATE user
            SET password = ?
            WHERE account = ?;
        """,
        (np, account))
        conn.commit()
        conn.close()
        Success(f"Successfully updated password for {account}")
    except:
        Error()

@click.command()
@click.password_option()
def get(password: str):
    verify(password)
    account = click.prompt("Account (gmail, reddit, facebook, etc)").lower()
    
    try:
        c, conn = connect()
        c.execute("""
            SELECT account, username, password FROM user 
            WHERE account = ?
            LIMIT 1; """,
        (account,))
        res = c.fetchone()
        click.echo(f"{res[0]} | {res[1]} | {decrypt(password, res[2])}")
        conn.commit()
        conn.close()
    except:
        Error()

@click.command()
@click.password_option()
def list(password):
    verify(password)

    try:
        c, conn = connect()
        c.execute("SELECT * FROM user;")
        res = c.fetchall()
        click.echo(res)
        conn.commit()
        conn.close()
    except:
        Error()



import click
from .account import reset, setup, wipe
from .queries import list, get, add, update

@click.group()
def cli():
    pass

cli.add_command(setup)
cli.add_command(reset)
cli.add_command(wipe)

cli.add_command(list)
cli.add_command(get)
cli.add_command(add)
cli.add_command(update)




import sqlite3
from cryptography.hazmat.primitives.kdf.scrypt import Scrypt
import os
from .messages import Error, ErrorWrongPassword
from cryptography.exceptions import InvalidKey
from .constant import secret_salt

def connect():
	conn = sqlite3.connect('management.db')
	c = conn.cursor()
	return c, conn

def genKDF():
	kdf = Scrypt(
		salt=secret_salt,
		length=32,
		n=2**14,
		r=8,
		p=1,
	)
	return kdf

def genHash(password):
	kdf = genKDF()
	return kdf.derive(bytes(password, "utf-8"))

def verify(password):
	try:
		c, conn = connect()
		c.execute("""
			SELECT password FROM user
			WHERE account = 'master';
		""")
		db_p = c.fetchone()[0]
		conn.commit()
		conn.close()

		kdf = genKDF()
		kdf.verify(bytes(password, 'utf-8'), db_p)
	except InvalidKey:
		raise ValueError("Wrong Password")
	except:
		raise Exception("Error")


from cryptography.fernet import Fernet
from .constant import secret_salt
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from .db import connect
import base64

def getKey(password: str):
	c, conn = connect()
	c.execute("""
		SELECT username FROM user
		WHERE account = 'master';
	""")
	salt = c.fetchone()[0]
	kdf = PBKDF2HMAC(
		algorithm=hashes.SHA256(),
		length=32,
		salt=salt,
		iterations=390000,
	)
	hashed = kdf.derive(bytes(password, "utf-8"))
	return base64.urlsafe_b64encode(hashed)

def encrypt(master_password, account_password):
	key = getKey(master_password)
	f = Fernet(key)
	token = f.encrypt(bytes(account_password, "utf-8"))
	return token

def decrypt(master_password, token_password):
	key = getKey(master_password)
	f = Fernet(key)
	res = f.decrypt(token_password)
	return res.decode("utf-8")


import click
def Error(m="An error occurred"):
	click.echo(click.style(m, fg="red"))
def ErrorWrongPassword():
	Error("Wrong Password.")
def Success(m = "Success!"):
	click.echo(click.style(m, fg="green"))


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['p3man']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'cryptography>=37.0.4,<38.0.0']

entry_points = \
{'console_scripts': ['p3man = p3man:cli']}

setup_kwargs = {
    'name': 'p3man',
    'version': '0.1.0',
    'description': 'Password manager. Manage your passwords through encryption.',
    'long_description': None,
    'author': 'curtis',
    'author_email': 'curtis.hu688@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
